@@ -169,7 +169,10 @@ class obj:
             Ny = self.GridLayer_Nxy_list[ptri][1]
             dN = 1./Nx/Ny
 
-            ep_grid = bd.reshape(ep_all[ptr:ptr+Nx*Ny],[Nx,Ny])
+            if len(ep_all) == 3 and ep_all[0].ndim>0:
+                ep_grid = [bd.reshape(ep_all[0][ptr:ptr+Nx*Ny],[Nx,Ny]),bd.reshape(ep_all[1][ptr:ptr+Nx*Ny],[Nx,Ny]),bd.reshape(ep_all[2][ptr:ptr+Nx*Ny],[Nx,Ny])]
+            else:
+                ep_grid = bd.reshape(ep_all[ptr:ptr+Nx*Ny],[Nx,Ny])
             
             epinv, ep2 = Epsilon_fft(dN,ep_grid,self.G)
 
@@ -186,6 +189,33 @@ class obj:
             ptr += Nx*Ny
             ptri += 1            
 
+    def Return_eps(self,which_layer,Nx,Ny,component='xx'):
+        '''
+        For patterned layer component = 'xx','xy','yx','yy','zz'
+        For uniform layer, currently it's assumed to be isotropic
+        '''
+        i = which_layer
+        # uniform layer
+        if self.id_list[i][0] == 0:
+            ep = self.Uniform_ep_list[self.id_list[i][2]]
+            return bd.ones((Nx,Ny))*ep
+
+        # patterned layer
+        elif self.id_list[i][0] == 1:
+            if component == 'zz':
+                epk = bd.inv(self.Patterned_epinv_list[self.id_list[i][2]])
+            elif component == 'xx':
+                epk = self.Patterned_ep2_list[self.id_list[i][2]][:self.nG,:self.nG]
+            elif component == 'xy':
+                epk = self.Patterned_ep2_list[self.id_list[i][2]][:self.nG,self.nG:]
+            elif component == 'yx':
+                epk = self.Patterned_ep2_list[self.id_list[i][2]][self.nG:,:self.nG]
+            elif component == 'yy':
+                epk = self.Patterned_ep2_list[self.id_list[i][2]][self.nG:,self.nG:]
+                
+            return get_ifft(Nx,Ny,epk[0,:],self.G)
+
+            
     def RT_Solve(self,normalize = 0, byorder = 0):
         '''
         Reflection and transmission power computation
@@ -219,7 +249,7 @@ class obj:
             ai = self.a0
             bi = b0
 
-        elif which_layer == self.Layer_N:
+        elif which_layer == self.Layer_N-1:
             aN, b0 = SolveExterior(self.a0,self.bN,self.q_list,self.phi_list,self.kp_list,self.thickness_list)
             ai = aN
             bi = self.bN
